/*	Copyright (C) 2003-2010 Free Electron Organization
	Any use of this software requires a license.  If a valid license
	was not distributed with this file, visit freeelectron.org. */

/** @file */

#ifndef __houdini_SurfaceAccessibleHoudini_h__
#define __houdini_SurfaceAccessibleHoudini_h__

namespace fe
{

/**************************************************************************//**
    @brief Houdini Surface Binding

	@ingroup houdini
*//***************************************************************************/
class FE_DL_EXPORT SurfaceAccessibleHoudini:
	public SurfaceAccessibleBase
{
	public:
								SurfaceAccessibleHoudini(void):
									m_pGdp(NULL)							{}

virtual							~SurfaceAccessibleHoudini(void)				{}

								//* as SurfaceAccessibleI
virtual	BWORD					isBound(void)	{ return m_pGdp!=NULL; }

virtual sp<SurfaceAccessorI>	accessor(SurfaceAccessibleI::Element a_element,
										String a_name,
										SurfaceAccessibleI::Creation a_create)
								{
									sp<SurfaceAccessorHoudini> spAccessor=
											accessor();
									if(spAccessor.isValid() &&
											(spAccessor->bind(a_element,
											a_name) || (m_pGdpChangeable &&
											a_create==
											SurfaceAccessibleI::e_createMissing)))
									{
										spAccessor->setSurfaceAccessible(
												sp<SurfaceAccessibleI>(this));
										return accessorOf(spAccessor);
									}
									return sp<SurfaceAccessorI>(NULL);
								}

virtual sp<SurfaceAccessorI>	accessor(SurfaceAccessibleI::Element a_element,
										SurfaceAccessibleI::Attribute
										a_attribute,
										SurfaceAccessibleI::Creation a_create)
								{
									sp<SurfaceAccessorHoudini> spAccessor=
											accessor();
									if(spAccessor.isValid() &&
											(spAccessor->bind(a_element,
											a_attribute) || (m_pGdpChangeable &&
											a_create==
											SurfaceAccessibleI::e_createMissing)))
									{
										spAccessor->setSurfaceAccessible(
												sp<SurfaceAccessibleI>(this));
										return accessorOf(spAccessor);
									}
									return sp<SurfaceAccessorI>(NULL);
								}

virtual	sp<SurfaceRange>		atomize(AtomicChange a_atomicChange);

virtual	sp<SurfaceI>			surface(void)
								{	return surface(String()); }
virtual	sp<SurfaceI>			surface(String a_group);
virtual	sp<SurfaceI>			subSurface(U32 a_subIndex)
								{	return subSurface(a_subIndex,String()); }
virtual	sp<SurfaceI>			subSurface(U32 a_subIndex,String a_group);

virtual	void					append(sp<SurfaceAccessibleI>
										a_spSurfaceAccessibleI,
										const SpatialTransform& a_rTransform);

virtual	void					instance(sp<SurfaceAccessibleI>
										a_spSurfaceAccessibleI,
										const std::vector<SpatialTransform>&
										a_rTransformArray);

								//* Houdini specific
		void					setGdp(const GU_Detail* a_pGdp)
								{
									safeLock();

									m_primitiveMap.clear();

//									if(m_pGdp!=a_pGdp || m_pGdpChangeable)
									{
										m_pGdp=a_pGdp;
										m_pGdpChangeable=NULL;
										m_spSurfaceI=NULL;
									}

									if(m_spSurfaceI.isValid())
									{
										sp<SurfaceTrianglesHoudini> spSurface=
												m_spSurfaceI;
										if(spSurface.isValid())
										{
											spSurface->setGdp(a_pGdp);
										}
									}

									safeUnlock();
								}
		void					setGdpChangeable(GU_Detail* a_pGdp)
								{
									safeLock();

									m_primitiveMap.clear();

//									if(m_pGdp!=a_pGdp ||
//											m_pGdpChangeable!=a_pGdp)
									{
										m_pGdp=a_pGdp;
										m_pGdpChangeable=a_pGdp;
										m_spSurfaceI=NULL;
									}

									if(m_spSurfaceI.isValid())
									{
										sp<SurfaceTrianglesHoudini> spSurface=
												m_spSurfaceI;
										if(spSurface.isValid())
										{
											spSurface->setGdp(a_pGdp);
										}
									}

									safeUnlock();
								}
const	GU_Detail*				gdp(void)			{ return m_pGdp; }
		GU_Detail*				gdpChangeable(void)	{ return m_pGdpChangeable; }

	private:
		sp<SurfaceRange>		atomizeCrossover(AtomicChange a_atomicChange);

		sp<SurfaceAccessorHoudini> accessor(void)
								{
									if(!m_pGdp)
									{
										return sp<SurfaceAccessorI>();
									}
									sp<SurfaceAccessorHoudini>
											spAccessorHoudini(
											new SurfaceAccessorHoudini);
									if(m_pGdpChangeable)
									{
										spAccessorHoudini->setGdpChangeable(
												m_pGdpChangeable);
									}
									else if(m_pGdp)
									{
										spAccessorHoudini->setGdp(m_pGdp);
									}

									return spAccessorHoudini;
								}
		sp<SurfaceAccessorI>	accessorOf(sp<SurfaceAccessorI> a_spAccessor)
								{
//#ifdef FE_HOUDINI_USE_GA
#if FALSE
									FEASSERT(a_spAccessor.isValid());
									if(!m_pGdpChangeable || threading()==
											SurfaceAccessibleI::e_singleThread)
									{
										return a_spAccessor;
									}

									sp<SurfaceAccessorCached> spAccessorCached=
											registry()->create(
											"*.SurfaceAccessorCached");
									FEASSERT(spAccessorCached.isValid());
									spAccessorCached->setSurfaceAccessible(
											sp<SurfaceAccessibleI>(this));
									spAccessorCached->setAccessorChain(
											a_spAccessor);
									return spAccessorCached;
#else
									return a_spAccessor;
#endif
								}

		void					initPrimitiveMap(void);
		I32						lookupPrimitive(I32 a_surfaceIndex) const;

const	GU_Detail*			m_pGdp;
		GU_Detail*			m_pGdpChangeable;

		sp<SurfaceI>		m_spSurfaceI;
		std::map<I32,I32>	m_primitiveMap;
		BWORD				m_hasDiscretePolygons;
};

} // namespace

#endif /* __houdini_SurfaceAccessibleHoudini_h__ */
