/*	Copyright (C) 2003-2010 Free Electron Organization
	Any use of this software requires a license.  If a valid license
	was not distributed with this file, visit freeelectron.org. */

/** @file */

#define FE_STM_DEBUG	FALSE

#include "maya.pmh"

namespace fe
{

SurfaceTrianglesMaya::SurfaceTrianglesMaya(void)
{
#if FE_STM_DEBUG
	feLog("SurfaceTrianglesMaya()\n");
#endif
}

SurfaceTrianglesMaya::~SurfaceTrianglesMaya(void)
{
}

Protectable* SurfaceTrianglesMaya::clone(Protectable* pInstance)
{
#if FE_STM_DEBUG
	feLog("SurfaceTrianglesMaya::clone\n");
#endif

	SurfaceTrianglesMaya* pSurfaceTrianglesMaya= pInstance?
			dynamic_cast<SurfaceTrianglesMaya*>(pInstance):
			new SurfaceTrianglesMaya();

	SurfaceTriangles::clone(pSurfaceTrianglesMaya);

	// TODO copy attributes

	return pSurfaceTrianglesMaya;
}

void SurfaceTrianglesMaya::cache(void)
{
#if FE_STM_DEBUG
	feLog("SurfaceTrianglesMaya::cache\n");
#endif

	sp<SurfaceAccessorMaya> spAccessor;
	if(!group().empty())
	{
		spAccessor=new SurfaceAccessorMaya;
		spAccessor->setMaster(registry()->master());
		spAccessor->setMeshObject(m_meshObject);
		spAccessor->bind(SurfaceAccessibleI::e_primitive,group());
	}

	//* NOTE not doing any Record serialization

	//* convert Maya to vertex and normal arrays

	MFnMesh* pFnMesh=new MFnMesh(m_meshObject);

	const U32 primitiveCount=pFnMesh->numPolygons();

	m_vertices=primitiveCount*3*2;	//* double in case they are quads
	resizeArrays(m_vertices);

	U32 total=0;
	for(U32 primitiveIndex=0;primitiveIndex<primitiveCount;primitiveIndex++)
	{
		//* if using group and not in that group
		if(spAccessor.isValid() && !spAccessor->integer(primitiveIndex))
		{
			continue;
		}

		const U32 subCount=pFnMesh->polygonVertexCount(primitiveIndex);
		if(subCount!=3 && subCount!=4)
		{
#if FE_STM_DEBUG
			feLog("SurfaceTrianglesMaya::cache"
					" primitive %d/%d subCount=%d\n",
					primitiveIndex,primitiveCount,subCount);
#endif
			continue;
		}

		MIntArray vertexArray;
		MStatus status=pFnMesh->getPolygonVertices(primitiveIndex,vertexArray);

		MFloatVectorArray normalArray;
		status=pFnMesh->getFaceVertexNormals(primitiveIndex,normalArray);
		I32 normalCount=normalArray.length();

		const U32 triIndex=(subCount==4)? primitiveIndex*2: primitiveIndex;

		for(U32 subIndex=0;subIndex<subCount;subIndex++)
		{
			MPoint point;
			status=pFnMesh->getPoint(vertexArray[subIndex],point);

			float u=0.0;
			float v=0.0;
			pFnMesh->getUV(vertexArray[subIndex],u,v);

			m_pVertexArray[total]=SpatialVector(point[0],point[1],point[2]);
			m_pPickIndexArray[total]=triIndex;
			m_pPrimitiveIndexArray[total]=primitiveIndex;
			m_pPartitionIndexArray[total]= -1;	//* TODO
			set(m_pUVArray[total],u,v);

			if(subIndex<normalCount)
			{
				MFloatVector& rNormal=normalArray[subIndex];
				m_pNormalArray[total]=
						SpatialVector(rNormal[0],rNormal[1],rNormal[2]);
			}
			else
			{
				m_pNormalArray[total]=SpatialVector(0.0,1.0,0.0);
			}

			total++;

			if(subIndex==3)
			{
					m_pVertexArray[total]=m_pVertexArray[total-4];
					m_pNormalArray[total]=m_pNormalArray[total-4];
					m_pUVArray[total]=m_pUVArray[total-4];
					m_pPickIndexArray[total]=triIndex+1;
					m_pPrimitiveIndexArray[total]=primitiveIndex;
					m_pPartitionIndexArray[total]= -1;	//* TODO
					total++;

					m_pVertexArray[total]=m_pVertexArray[total-3];
					m_pNormalArray[total]=m_pNormalArray[total-3];
					m_pUVArray[total]=m_pUVArray[total-3];
					m_pPickIndexArray[total]=triIndex+1;
					m_pPrimitiveIndexArray[total]=primitiveIndex;
					m_pPartitionIndexArray[total]= -1;	//* TODO
					total++;
			}
		}
	}

	delete pFnMesh;

	if(total!=m_vertices)
	{
#if FE_STM_DEBUG
		feLog("SurfaceTrianglesMaya::cache"
				" converted %d/%d vertices\n",total,m_vertices);
#endif

		m_vertices=total;
		resizeArrays(m_vertices);
	}

	calcBoundingSphere();
}

} // namespace
