/*	Copyright (C) 2003-2010 Free Electron Organization
	Any use of this software requires a license.  If a valid license
	was not distributed with this file, visit freeelectron.org. */

/** @file */

#define FE_OST_IO_DEBUG		FALSE

#include "operate/operate.pmh"

using namespace fe;

void OperatorState::initialize(void)
{
	m_spScope=registry()->create("Scope");
	m_spScope->setName(name()+".State");
	m_spState=new RecordGroup();

	setupState();

	catalog<String>("IdAttr")="id";
	catalog<String>("IdAttr","label")="ID Attribute";
	catalog<String>("IdAttr","page")="Data";
	catalog<String>("IdAttr","hint")=
			"If defined, store changes based on this an identifier"
			" in this primitive attribute instead of"
			" the index of simple primitive order."
			"  The values should be unique integers.";
}

void OperatorState::updateIds(void)
{
	const String idAttr=catalog<String>("IdAttr");
	sp<SurfaceAccessorI>	m_spInputIds;
	m_spInputIds=NULL;
	if(!idAttr.empty())
	{
		access(m_spInputIds,"Input Surface",e_primitive,idAttr,e_quiet);
	}

	m_indexMap.clear();
	const U32 count=m_spInputIds->count();
	if(m_spInputIds.isValid())
	{
		for(U32 primIndex=0;primIndex<count;primIndex++)
		{
			m_indexMap[m_spInputIds->integer(primIndex)]=primIndex;
		}
	}
}

I32 OperatorState::lookupIndex(I32 a_id)
{
	if(a_id<0 || m_spInputIds.isNull())
	{
		return a_id;
	}
	if(m_indexMap.find(a_id)==m_indexMap.end())
	{
		return -1;
	}
	return m_indexMap[a_id];
}

I32 OperatorState::lookupId(I32 a_index)
{
	return m_spInputIds.isValid()?  m_spInputIds->integer(a_index): a_index;
}

BWORD OperatorState::loadState(const String& rBuffer)
{
#if FE_OST_IO_DEBUG
	feLog("OperatorState::loadState\n");
	feLog("---------\n");
	feLog("%s",rBuffer.c_str());
	feLog("---------\n");
#endif

	std::string stringIn=rBuffer.c_str();
	std::istringstream streamIn(stringIn);

	sp<data::StreamI> spStream(new data::AsciiStream(m_spScope));
	sp<RecordGroup> spLoadedGroup=spStream->input(streamIn);

	if(spLoadedGroup.isValid())
	{
		m_spState=spLoadedGroup;
	}

	return TRUE;
}

BWORD OperatorState::saveState(String& rBuffer)
{
#if FE_OST_IO_DEBUG
	feLog("OperatorState::saveState\n");
#endif

	sp<data::StreamI> spStream(new data::AsciiStream(m_spScope));
	std::ostringstream streamOut;
	spStream->output(streamOut,m_spState);

	std::stringbuf* pStringBuf=streamOut.rdbuf();
	rBuffer=pStringBuf->str().c_str();

#if FE_OST_IO_DEBUG
	feLog("---------\n");
	feLog("%s",rBuffer.c_str());
	feLog("---------\n");
#endif

	return TRUE;
}

BWORD OperatorState::pickToggle(I32 a_index)
{
	const I32 pickCount=m_pickArray.size();
	for(I32 pickIndex=0;pickIndex<pickCount;pickIndex++)
	{
		const Pick& rPick=m_pickArray[pickIndex];
		const I32 pointIndex=rPick.m_index;
		if(a_index==pointIndex)
		{
			m_pickArray.erase(m_pickArray.begin()+pickIndex);
			return TRUE;
		}
	}
	pick(a_index);
	return FALSE;
}

void OperatorState::pick(I32 a_index,Real a_weight)
{
	const I32 pickCount=m_pickArray.size();
	for(I32 pickIndex=0;pickIndex<pickCount;pickIndex++)
	{
		Pick& rPick=m_pickArray[pickIndex];
		const I32 pointIndex=rPick.m_index;
		if(a_index==pointIndex)
		{
			if(rPick.m_weight<a_weight)
			{
				rPick.m_weight=a_weight;
			}
			return;
		}
	}

	m_pickArray.push_back(Pick(a_index,a_weight));
}

Real OperatorState::picked(I32 a_index)
{
	const I32 pickCount=m_pickArray.size();
	for(I32 pickIndex=0;pickIndex<pickCount;pickIndex++)
	{
		const Pick& rPick=m_pickArray[pickIndex];
		const I32 pointIndex=rPick.m_index;
		if(a_index==pointIndex)
		{
			return rPick.m_weight;
		}
	}
	return 0.0;
}

