/*	Copyright (C) 2003-2010 Free Electron Organization
	Any use of this software requires a license.  If a valid license
	was not distributed with this file, visit freeelectron.org. */

/** @file */

#ifndef __operator_CurvaceousOp_h__
#define __operator_CurvaceousOp_h__

namespace fe
{

/**************************************************************************//**
	@brief Operator to interactively alter curves

	@ingroup operator
*//***************************************************************************/
class FE_DL_EXPORT CurvaceousOp:
	public OperatorState,
	public Initialize<CurvaceousOp>
{
	public:

						CurvaceousOp(void):
							m_brushed(FALSE),
							m_dragging(FALSE),
							m_lastFrame(0.0),
							m_pickId(-1),
							m_pickVertex(-1),
							m_pickU(0.0)									{}

virtual					~CurvaceousOp(void)									{}

		void			initialize(void);

						//* As BindI
virtual	void			bind(sp<SignalerI> a_spSignalerI,
								sp<Layout> a_spLayout);

						//* As HandlerI
virtual	void			handle(Record& a_rSignal);

	protected:

virtual	void			setupState(void);
virtual	BWORD			loadState(const String& rBuffer);

	private:

		void			movePicked(const SpatialVector& a_rDelta);

		sp<OperatorSurfaceI>		m_spLengthCorrect;

		sp<SurfaceI>				m_spOutput;
		sp<SurfaceI>				m_spDriver;
		sp<SurfaceAccessorI>		m_spInputIds;
		sp<SurfaceAccessorI>		m_spInputPoint;
		sp<SurfaceAccessorI>		m_spOutputPoint;
		sp<SurfaceAccessorI>		m_spInputVertices;
		sp<SurfaceAccessorI>		m_spOutputVertices;
		WindowEvent					m_event;

		sp<DrawMode>				m_spSolid;

		sp< RecordMap<I32> >		m_spEditMap;
		Accessor<SpatialVector>		m_aOffset;

		Real						m_frame;
		Real						m_lastFrame;

		BWORD						m_brushed;
		BWORD						m_dragging;

		I32							m_pickId;
		I32							m_pickVertex;
		Real						m_pickU;

		I32							m_lastX;
		I32							m_lastY;
};

} /* namespace */

#endif /* __operator_CurvaceousOp_h__ */
