/*	Copyright (C) 2003-2010 Free Electron Organization
	Any use of this software requires a license.  If a valid license
	was not distributed with this file, visit freeelectron.org. */

#include "operator/operator.pmh"

#define FE_SSP_DEBUG	FALSE

namespace fe
{

void SurfaceSummaryOp::initialize(void)
{
	catalog<String>("icon")="FE_alpha";

	catalog<I32>("pointCount");
	catalog<I32>("pointCount","max")=INT_MAX;
	catalog<String>("pointCount","IO")="output";
	catalog<String>("pointCount","label")="Points (output)";

	catalog<I32>("primitiveCount");
	catalog<I32>("primitiveCount","max")=INT_MAX;
	catalog<String>("primitiveCount","IO")="output";
	catalog<String>("primitiveCount","label")="Primitives (output)";

	catalog<String>("probeAttribute")="P";
	catalog<String>("probeAttribute","label")="Probe Attribute";

	catalog<String>("probeClass")="Point";
	catalog<String>("probeClass","choice:0")="Point";
	catalog<String>("probeClass","choice:1")="Primitives";
	catalog<String>("probeClass","label")="Probe Class";

	catalog<I32>("probeIndex")=0;
	catalog<I32>("probeIndex","high")=100;
	catalog<I32>("probeIndex","max")=INT_MAX;
	catalog<String>("probeIndex","label")="Probe Index";

	catalog<String>("probeValue");
	catalog<String>("probeValue","IO")="output";
	catalog<String>("probeValue","label")="Probe (output)";

	catalog< sp<Component> >("Input Surface");
}

void SurfaceSummaryOp::handle(Record& a_rSignal)
{
#if FE_SSP_DEBUG
	feLog("SurfaceSummaryOp::handle\n");
#endif

	I32& pointCount=catalog<I32>("pointCount");
	I32& primitiveCount=catalog<I32>("primitiveCount");

	const String probeClass=catalog<String>("probeClass");
	const String probeAttribute=catalog<String>("probeAttribute");
	const I32 probeIndex=catalog<I32>("probeIndex");
	String& probeValue=catalog<String>("probeValue");

	sp<SurfaceAccessibleI> spInputSurface;
	if(!access(spInputSurface,"Input Surface")) return;

	sp<SurfaceAccessorI> spInputElement;
	access(spInputElement,spInputSurface,e_point,e_generic,e_warning);

	pointCount=(spInputElement.isValid())? spInputElement->count(): -1;

#if FE_SSP_DEBUG
	feLog("SurfaceSummaryOp::handle pointCount=%d\n",pointCount);
#endif

	access(spInputElement,spInputSurface,e_primitive,e_generic,e_warning);

	primitiveCount=(spInputElement.isValid())? spInputElement->count(): -1;

#if FE_SSP_DEBUG
	feLog("SurfaceSummaryOp::handle primitiveCount=%d\n",primitiveCount);
#endif

	bool probePoint=(probeClass=="Point");
	I32 probeMax=probePoint? pointCount: primitiveCount;

	if(probeIndex>=0 && probeIndex<probeMax)
	{
		probeValue="inaccessible";

		access(spInputElement,spInputSurface,
				probePoint? e_point: e_primitive,probeAttribute,e_warning);

		if(spInputElement.isValid())
		{
			const String probeType=spInputElement->type();
			probeValue="bad type";

			if(probeType=="String")
			{
				probeValue=spInputElement->string(probeIndex,0);
			}
			else if(probeType=="Integer")
			{
				probeValue.sPrintf("%d",spInputElement->integer(probeIndex,0));
			}
			else if(probeType=="Real")
			{
				probeValue.sPrintf("%.6G",spInputElement->real(probeIndex,0));
			}
			else if(probeType.match("Vector."))
			{
				probeValue.sPrintf("%s",
						c_print(spInputElement->spatialVector(probeIndex,0)));
			}
#if FE_SSP_DEBUG
			feLog("SurfaceSummaryOp::handle probeValue=%.6G\n",
					probeValue);
#endif
		}
	}
	else
	{
		probeValue="bad index";
	}

#if FE_SSP_DEBUG
	feLog("SurfaceSummaryOp::handle done\n");
#endif
}

} // namespace
