/*	Copyright (C) 2003-2010 Free Electron Organization
	Any use of this software requires a license.  If a valid license
	was not distributed with this file, visit freeelectron.org. */

/** @file */

#ifndef __surface_SurfaceAccessibleBase_h__
#define __surface_SurfaceAccessibleBase_h__

namespace fe
{

/**************************************************************************//**
    @brief Common Functionality for Accessible Surface

	@ingroup surface
*//***************************************************************************/
class FE_DL_EXPORT SurfaceAccessibleBase:
	public ObjectSafeShared<SurfaceAccessibleBase>,
	virtual public SurfaceAccessibleI
{
	public:
								SurfaceAccessibleBase(void)					{}
virtual							~SurfaceAccessibleBase(void)				{}

								//* as SurfaceAccessibleI

virtual	void					bind(sp<SurfaceAccessibleI::ThreadingState>
										a_spThreadingState)
								{	m_spThreadingState=a_spThreadingState; }
virtual	void					lock(void)		{ safeLock(); }
virtual	void					unlock(void)	{ safeUnlock(); }

virtual sp<SurfaceAccessorI>	accessor(Element a_element,String a_name,
										Creation a_create)
								{	return sp<SurfaceAccessorI>(NULL); }

virtual sp<SurfaceAccessorI>	accessor(Element a_element,
										Attribute a_attribute,
										Creation a_create)
								{	return sp<SurfaceAccessorI>(NULL); }

virtual sp<SurfaceAccessorI>	accessor(Element a_element,String a_name)
								{	return accessor(a_element,a_name,
											e_createMissing); }

virtual sp<SurfaceAccessorI>	accessor(Element a_element,
										Attribute a_attribute)
								{	return accessor(a_element,a_attribute,
											e_createMissing); }

								//* default implementation is non-atomic
virtual	sp<SurfaceRange>		atomize(AtomicChange a_atomicChange)
								{
									const Element element=
											a_atomicChange<e_primitivesOnly?
											e_point: e_primitive;
									sp<SurfaceAccessorI> spAccessor=
											accessor(element,e_generic);
									sp<SurfaceRange> spSurfaceRange(
											new SurfaceRange());
									if(spAccessor.isValid())
									{
										spSurfaceRange->nonAtomic().add(
												0,spAccessor->count()-1);
									}
									return spSurfaceRange;
								}

	protected:
		SurfaceAccessibleI::Threading	threading(void)
							{
								FEASSERT(m_spThreadingState.isValid());
								return Threading(
										m_spThreadingState->threading());
							}

		sp<SurfaceAccessibleI::ThreadingState>	m_spThreadingState;
};

} // namespace

#endif /* __surface_SurfaceAccessibleBase_h__ */
