/*	Copyright (C) 2003-2010 Free Electron Organization
	Any use of this software requires a license.  If a valid license
	was not distributed with this file, visit freeelectron.org. */

/** @file */

#include "surface.pmh"

namespace fe
{

SurfaceCurves::SurfaceCurves(void)
{
	m_pImpactPool=&m_curveImpactPool;
}

Protectable* SurfaceCurves::clone(Protectable* pInstance)
{
	feLog("SurfaceCurves::clone\n");

	SurfaceCurves* pSurfaceCurve= pInstance?
			dynamic_cast<SurfaceCurves*>(pInstance): new SurfaceCurves();

	checkCache();

	//* TODO

	return pSurfaceCurve;
}

void SurfaceCurves::cache(void)
{
	if(!m_record.isValid())
	{
		feLog("SurfaceCurves::cache record invalid\n");
		return;
	}

	SurfaceSphere::cache();

	//* TODO

	SurfaceModel surfaceModelRV;
	surfaceModelRV.bind(m_record);

	sp<RecordGroup> spRG=surfaceModelRV.surfacePointRG();
	for(RecordGroup::iterator it=spRG->begin();it!=spRG->end();it++)
	{
		sp<RecordArray> spRA= *it;

		SurfacePoint m_surfacePointRV;
		m_surfacePointRV.bind(spRA);
		while(m_surfacePointRV.step())
		{
			SpatialVector& location=m_surfacePointRV.location();
			SpatialVector& up=m_surfacePointRV.up();

			m_pVertexArray=(SpatialVector*)reallocate(m_pVertexArray,
					(m_vertices+1)*sizeof(SpatialVector));

			SpatialVector& rVertex=m_pVertexArray[m_vertices];

			rVertex=location;

			printf("vertex %d  %s\n",m_vertices,c_print(rVertex));
			m_vertices++;
		}
	}
}

//* TODO convert triangle logic to segment logic
SpatialTransform SurfaceCurves::sample(I32 a_face,
	SpatialBary a_barycenter,SpatialVector a_tangent) const
{
	feLog("SurfaceCurves::sample prim %d bary %s tan %s\n",
			a_face,c_print(a_barycenter),c_print(a_tangent));

	//* TODO map
	const U32 triCount=m_vertices/3;
	I32 triIndex;
	for(triIndex=0;triIndex<triCount;triIndex++)
	{
		if(m_pPickIndexArray[triIndex*3]==a_face)
		{
			break;
		}
	}

	const I32 vertIndex=a_face*m_grain;
	if(vertIndex+m_grain>m_vertices)
	{
#if TRUE
		//* WARNING multi-threaded logging can segfault
		feLog("SurfaceCurves::sample"
				" excess face %d (%d>%d)\n",
				a_face,vertIndex+m_grain,m_vertices);
#endif
		return SpatialTransform::identity();
	}
	const I32 edgeIndex=vertIndex+a_barycenter[1]*m_grain;

	const SpatialVector& rVertex0=m_pVertexArray[edgeIndex];
	const SpatialVector& rVertex1=m_pVertexArray[edgeIndex+1];
	const SpatialVector& rNormal0=m_pNormalArray[edgeIndex];
	const SpatialVector& rNormal1=m_pNormalArray[edgeIndex+1];

	const SpatialVector point=rVertex0*a_barycenter[0]+rVertex1*a_barycenter[1];
	const SpatialVector norm=rNormal0*a_barycenter[0]+rNormal1*a_barycenter[1];
	const Real mag=magnitude(norm);
	SpatialTransform result;
	if(mag>0.0)
	{
		SpatialVector tangent=a_tangent;

		if(m_pUVArray)
		{
			const SpatialVector& rUV0=m_pUVArray[edgeIndex];
			const SpatialVector& rUV1=m_pUVArray[edgeIndex+1];

			SpatialVector du;
			SpatialVector dv;
			triangleDuDv(du,dv,rVertex0,rVertex1,rVertex1,rUV0,rUV1,rUV1);

			tangent=du;

			feLog("du %s dv %s\n",c_print(du),c_print(dv));
		}
		if(magnitudeSquared(tangent)<1e-6)
		{
			tangent=rVertex1-rVertex0;
		}
		makeFrameNormalY(result,point,tangent,norm/mag);

		feLog("a_tangent %s tangent %s\n",c_print(a_tangent),c_print(tangent));
	}
	else
	{
		setIdentity(result);
		setTranslation(result,point);
	}

#if TRUE
	feLog("face %d/%d bary %s\n",a_face,m_vertices/m_grain,
			c_print(a_barycenter));
	feLog("rVertex0 %s\n",c_print(rVertex0));
	feLog("rVertex1 %s\n",c_print(rVertex1));
	feLog("rNormal0 %s\n",c_print(rNormal0));
	feLog("rNormal1 %s\n",c_print(rNormal1));
	feLog("point %s\n",c_print(point));
	feLog("norm %s\n",c_print(norm));
	feLog("result %s\n",c_print(result));
#endif

	return result;
}


SpatialTransform SurfaceCurves::sample(Vector2 a_uv) const
{
	SpatialTransform transform;

	//* TODO
	setIdentity(transform);

	return transform;
}

void SurfaceCurves::draw(const SpatialTransform& a_transform,
	sp<DrawI> a_spDrawI,const Color* color) const
{
	//* TODO
}

void SurfaceCurves::resolveImpact(sp<ImpactI> a_spImpactI) const
{
	sp<Impact> spImpact(a_spImpactI);

	// TEMP if no nearest
	if(!spImpact.isValid() || spImpact->distance()<Real(0))
	{
		return;
	}

	SpatialVector normal;
	set(normal,1.0,0.0,0.0);	//* TODO

	spImpact->setNormalLocal(normal);
}

} // namespace
