/*	Copyright (C) 2003-2010 Free Electron Organization
	Any use of this software requires a license.  If a valid license
	was not distributed with this file, visit freeelectron.org. */

/** @file */

#include "surface.pmh"

namespace fe
{

void SurfaceRange::Iterator::advance(void)
{
	m_spanIndex++;
	m_valueIndex=0;

	MultiSpan* pMultiSpan=(m_atomicIndex==m_atomicCount)?
			&m_spSurfaceRange->nonAtomic():
			&m_spSurfaceRange->atomic(m_atomicIndex);

	U32 spanCount=pMultiSpan->count();
	if(m_spanIndex==spanCount)
	{
		m_atomicIndex++;
		m_spanIndex=0;

		if(m_atomicIndex>m_atomicCount)
		{
			return;
		}

		pMultiSpan=(m_atomicIndex==m_atomicCount)?
				&m_spSurfaceRange->nonAtomic():
				&m_spSurfaceRange->atomic(m_atomicIndex);
		spanCount=pMultiSpan->count();

		if(!spanCount)
		{
			FEASSERT(m_atomicIndex==m_atomicCount);
			m_atomicIndex++;
			return;
		}
	}

	const Span& rSpan=pMultiSpan->span(m_spanIndex);

	m_value=rSpan[0];
	m_valueCount=rSpan[1]-rSpan[0]+1.0;
}

String SurfaceRange::MultiSpan::brief(void) const
{
	if(!m_spanArray.size())
	{
		return "<empty>";
	}

	String result;
	const U32 spanCount=m_spanArray.size();
	for(U32 m=0;m<spanCount;m++)
	{
		if(m)
		{
			result.cat(" ");
		}
		const I32 start=m_spanArray[m][0];
		const I32 end=m_spanArray[m][1];
		if(start==end)
		{
			result.cat("%d",I32(m_spanArray[m][0]));
		}
		else
		{
			result.cat("%d-%d",I32(m_spanArray[m][0]),I32(m_spanArray[m][1]));
		}
	}
	return result;

	//* TODO overall min,max if way too many spans
}

String SurfaceRange::MultiSpan::dump(void) const
{
	String result;
	const U32 spanCount=m_spanArray.size();
	for(U32 m=0;m<spanCount;m++)
	{
		result.cat("  span %d: %s\n",m,
			c_print(m_spanArray[m]));
	}
	return result;
}

String SurfaceRange::brief(void) const
{
	String result;
	const U32 atomicCount=m_atomArray.size();
	if(atomicCount==1)
	{
		result.sPrintf("[atomic %s, serial %s]",
				m_atomArray[0].brief().c_str(),
				m_nonAtomic.brief().c_str());
	}
	else
	{
		result.sPrintf("[%d atoms, serial %s]",m_atomArray.size(),
				m_nonAtomic.brief().c_str());
	}
	return result;
}

String SurfaceRange::dump(void) const
{
	String result;
	const U32 atomCount=m_atomArray.size();
	for(U32 m=0;m<atomCount;m++)
	{
		result.cat("atom %d:\n%s",m,
				m_atomArray[m].dump().c_str());
	}
	result.cat("non-atomic:\n%s",
			m_nonAtomic.dump().c_str());
	return result;
}

} // namespace
