/*	Copyright (C) 2003-2010 Free Electron Organization
	Any use of this software requires a license.  If a valid license
	was not distributed with this file, visit freeelectron.org. */

/** @file */

#ifndef __surface_SurfaceRange_h__
#define __surface_SurfaceRange_h__

namespace fe
{

/**************************************************************************//**
	@brief SurfaceRange of jobs

	@ingroup surface
*//***************************************************************************/
class FE_DL_EXPORT SurfaceRange: public Counted
{
	public:

	class Span: public Vector<2,Real>
	{
		public:
					Span(void):
						Vector<2,Real>(0.0,-1.0)							{}
	};

	class MultiSpan
	{
		public:
								MultiSpan(void)								{}

			void				clear(void)
								{	m_spanArray.clear(); }
			U32 				count(void) const
								{	return m_spanArray.size(); }
	const	Span&				span(U32 a_index) const
								{
									FEASSERT(a_index<m_spanArray.size());
									return m_spanArray[a_index];
								}

								//* TODO sort
								//* TODO merge with existing
			void				add(Real a_start,Real a_end)
								{
									const U32 index=m_spanArray.size();
									m_spanArray.resize(index+1);
									set(m_spanArray[index],a_start,a_end);
								}

			String				brief(void) const;
			String				dump(void) const;

			std::vector<Span>	m_spanArray;
	};

	class Iterator
	{
		public:
								Iterator(sp<SurfaceRange> a_spSurfaceRange):
									m_spSurfaceRange(a_spSurfaceRange),
									m_atomicIndex(0),
									m_spanIndex(-1),
									m_valueCount(0),
									m_valueIndex(0),
									m_value(0.0)
								{
									FEASSERT(m_spSurfaceRange.isValid());
									m_atomicCount=
											m_spSurfaceRange->atomicCount();
									advance();
								}

			Real				value(void)		{ return m_value; }
			BWORD				atEnd(void)
								{	return m_atomicIndex>m_atomicCount; }

			void				step(void)
								{
									m_valueIndex++;
									if(m_valueIndex>=m_valueCount)
									{
										advance();
									}
									else
									{
										m_value+=1.0;
									}
								}

		private:

			void				advance(void);

			sp<SurfaceRange>	m_spSurfaceRange;
			U32					m_atomicCount;
			U32					m_atomicIndex;
			I32					m_spanIndex;
			U32					m_valueCount;
			U32					m_valueIndex;
			Real				m_value;
	};

					SurfaceRange(void)										{}

		void		clear(void)
					{
						m_atomArray.clear();
						m_nonAtomic.clear();
					}

		MultiSpan&	addAtomic(void)
					{
						const U32 index=m_atomArray.size();
						m_atomArray.resize(index+1);
						return m_atomArray[index];
					}
		U32			atomicCount(void)		{ return m_atomArray.size(); }
		MultiSpan&	atomic(U32 a_index)		{ return m_atomArray[a_index]; }
		MultiSpan&	nonAtomic(void)			{ return m_nonAtomic; }

		BWORD		empty(void) const
					{	return !m_atomArray.size() && !m_nonAtomic.count(); }

		Iterator	begin(void)	{ return Iterator(sp<SurfaceRange>(this)); }

		String		brief(void) const;
		String		dump(void) const;

	private:

		std::vector<MultiSpan>	m_atomArray;	//* surface-safe units
		MultiSpan				m_nonAtomic;	//* not atomic (already safe)
};

} // namespace

#endif // __surface_SurfaceRange_h__
