/*	Copyright (C) 2003-2010 Free Electron Organization
	Any use of this software requires a license.  If a valid license
	was not distributed with this file, visit freeelectron.org. */

/** @file */

#ifndef __surface_SurfaceTriangles_h__
#define __surface_SurfaceTriangles_h__

namespace fe
{

/**************************************************************************//**
	@brief Discrete Triangle Surface

	@ingroup surface
*//***************************************************************************/
class FE_DL_EXPORT SurfaceTriangles: public SurfaceSearchable
{
	public:

	class Impact: public SurfaceSearchable::Impact
	{
		public:
								Impact(void)
								{
#if FE_COUNTED_STORE_TRACKER
									setName("SurfaceTriangles::Impact");
#endif
								}
	virtual						~Impact(void)								{}

	virtual	SpatialVector		vertex0(void) const
								{	return m_vertex0; }
			void				setVertex0(SpatialVector a_vertex0)
								{	m_vertex0=a_vertex0; }

	virtual	SpatialVector		vertex1(void) const
								{	return m_vertex1; }
			void				setVertex1(SpatialVector a_vertex1)
								{	m_vertex1=a_vertex1; }

	virtual	SpatialVector		vertex2(void) const
								{	return m_vertex2; }
			void				setVertex2(SpatialVector a_vertex2)
								{	m_vertex2=a_vertex2; }

	virtual	SpatialVector		normal0(void) const
								{	return m_normal; }
			void				setNormal0(SpatialVector a_normal0)
								{	m_normal=a_normal0; }

	virtual	SpatialVector		normal1(void) const
								{	return m_normal1; }
			void				setNormal1(SpatialVector a_normal1)
								{	m_normal1=a_normal1; }

	virtual	SpatialVector		normal2(void) const
								{	return m_normal2; }
			void				setNormal2(SpatialVector a_normal2)
								{	m_normal2=a_normal2; }

	virtual	I32					face(void)
								{	return m_face; }
			void				setFace(I32 a_face)
								{	m_face=a_face; }

	virtual	void				copy(sp<SpatialTreeI::Hit>& a_rspHit);

		private:
			void				resolve(void)
								{
//									SurfaceSphere::Impact::resolve();

									const SurfaceTriangles* pSurfaceTriangles=
											dynamic_cast
											<const SurfaceTriangles*>(
											m_pSurface);
									pSurfaceTriangles->resolveImpact(
											sp<ImpactI>(this));
									m_resolved=TRUE;
								}

			SpatialVector			m_vertex0;
			SpatialVector			m_vertex1;
			SpatialVector			m_vertex2;
			SpatialVector			m_normal1;
			SpatialVector			m_normal2;
			I32						m_face;
	};

							SurfaceTriangles(void);
virtual						~SurfaceTriangles(void);

							//* As SurfaceI
virtual SpatialTransform	sample(I32 a_face,SpatialBary a_barycenter,
									SpatialVector a_tangent) const;

virtual	std::vector< sp<SurfaceI::ImpactI> >
							nearestPoints(const SpatialVector& a_origin,
									U32 a_hitLimit) const;
virtual	sp<ImpactI>			nearestPoint(const SpatialVector& a_origin) const;

virtual	std::vector< sp<SurfaceI::ImpactI> >
							rayImpacts(const SpatialVector& a_origin,
									const SpatialVector& a_direction,
									Real a_maxDistance,U32 a_hitLimit) const;
virtual	sp<ImpactI>			rayImpact(const SpatialVector& a_origin,
									const SpatialVector& a_direction,
									Real a_maxDistance) const;

virtual	void				draw(sp<DrawI> a_spDrawI,
									const Color* a_pColor) const
							{	drawInternal(FALSE,NULL,a_spDrawI,a_pColor); }
virtual	void				draw(const SpatialTransform& a_rTransform,
									sp<DrawI> a_spDrawI,
									const fe::Color* a_pColor) const
							{	drawInternal(TRUE,&a_rTransform,
										a_spDrawI,a_pColor); }
virtual	void				drawInternal(BWORD a_transformed,
									const SpatialTransform* a_pTransform,
									sp<DrawI> a_spDrawI,
									const fe::Color* a_pColor) const;

		void				setGroup(String a_group)
							{	m_group=a_group; }
		String				group(void) const
							{	return m_group; }

	protected:

virtual	void				resolveImpact(sp<ImpactI> a_spImpactI) const;

	private:
mutable	CountedPool<Impact>	m_triangleImpactPool;
		String				m_group;
};

} // namespace

#endif /* __surface_SurfaceTriangles_h__ */
